from helpers import compute_volume

import pygmsh


def test_geo():
    with pygmsh.geo.Geometry() as geom:
        poly = geom.add_polygon(
            [
                [0.0, 0.0, 0.0],
                [2.0, 0.0, 0.0],
                [3.0, 1.0, 0.0],
                [1.0, 2.0, 0.0],
                [0.0, 1.0, 0.0],
            ],
            mesh_size=0.1,
        )

        field0 = geom.add_boundary_layer(
            edges_list=[poly.curve_loop.curves[0]],
            lcmin=0.01,
            lcmax=0.1,
            distmin=0.0,
            distmax=0.2,
        )
        field1 = geom.add_boundary_layer(
            nodes_list=[poly.curve_loop.curves[1].points[1]],
            lcmin=0.01,
            lcmax=0.1,
            distmin=0.0,
            distmax=0.2,
        )
        geom.set_background_mesh([field0, field1], operator="Min")

        ref = 4.0
        mesh = geom.generate_mesh()
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref

    return mesh


def test_occ():
    with pygmsh.occ.Geometry() as geom:
        geom.add_rectangle([0.0, 0.5, 0.0], 5.0, 0.5)

        edge1 = pygmsh.occ.dummy.Dummy(dim=1, id0=1)
        point1 = pygmsh.occ.dummy.Dummy(dim=0, id0=3)

        field0 = geom.add_boundary_layer(
            edges_list=[edge1],
            lcmin=0.01,
            lcmax=0.1,
            distmin=0.0,
            distmax=0.2,
            num_points_per_curve=50,
        )
        field1 = geom.add_boundary_layer(
            nodes_list=[point1],
            lcmin=0.01,
            lcmax=0.1,
            distmin=0.0,
            distmax=0.2,
            num_points_per_curve=50,
        )
        geom.set_background_mesh([field0, field1], operator="Min")

        ref = 2.5
        mesh = geom.generate_mesh()
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test_geo().write("boundary_layers_geo.vtu")
    test_occ().write("boundary_layers_occ.vtu")



import pytest


def test_raise_duplicate():
    import pygmsh

    with pygmsh.geo.Geometry() as geom:
        p = geom.add_rectangle(-1, 1, -1, 1, z=0, mesh_size=1)
        geom.add_physical(p.lines[0], label="A")
        with pytest.raises(ValueError):
            geom.add_physical(p.lines[1], label="A")



from math import pi

from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.add_ellipsoid([1.0, 1.0, 1.0], [1.0, 2.0, 3.0], mesh_size=0.1)
        mesh = geom.generate_mesh()

    ref = 8.0 * pi
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("occ_ellipsoid.vtu")



from math import pi

from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.add_cone(
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 1.0],
            1.0,
            0.3,
            mesh_size=0.1,
            angle=1.25 * pi,
        )
        mesh = geom.generate_mesh()

    ref = 0.90779252263
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("occ_cone.vtu")



from math import pi

from helpers import compute_volume

import pygmsh


def test(mesh_size=0.05):
    with pygmsh.geo.Geometry() as geom:
        # Draw a square
        poly = geom.add_polygon(
            [
                [+0.5, +0.0, 0.0],
                [+0.0, +0.5, 0.0],
                [-0.5, +0.0, 0.0],
                [+0.0, -0.5, 0.0],
            ],
            mesh_size=mesh_size,
        )
        axis = [0, 0, 1.0]
        geom.twist(
            poly,
            translation_axis=axis,
            rotation_axis=axis,
            point_on_axis=[0.0, 0.0, 0.0],
            angle=0.5 * pi,
            num_layers=5,
            recombine=True,
        )
        mesh = geom.generate_mesh()

    ref = 3.98156496566
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("rotated_layers.vtu")



from helpers import compute_volume
from numpy import cos, pi, sin

import pygmsh


def test(lcar=0.3):
    with pygmsh.geo.Geometry() as geom:
        r = 1.25 * 3.4
        p1 = geom.add_point([0.0, 0.0, 0.0], lcar)
        # p2 = geom.add_point([+r, 0.0, 0.0], lcar)
        p3 = geom.add_point([-r, 0.0, 0.0], lcar)
        p4 = geom.add_point([0.0, +r, 0.0], lcar)
        p5 = geom.add_point([0.0, -r, 0.0], lcar)
        p6 = geom.add_point([r * cos(+pi / 12.0), r * sin(+pi / 12.0), 0.0], lcar)
        p7 = geom.add_point([r * cos(-pi / 12.0), r * sin(-pi / 12.0), 0.0], lcar)
        p8 = geom.add_point([0.5 * r, 0.0, 0.0], lcar)

        c0 = geom.add_circle_arc(p6, p1, p4)
        c1 = geom.add_circle_arc(p4, p1, p3)
        c2 = geom.add_circle_arc(p3, p1, p5)
        c3 = geom.add_circle_arc(p5, p1, p7)
        l1 = geom.add_line(p7, p8)
        l2 = geom.add_line(p8, p6)
        ll = geom.add_curve_loop([c0, c1, c2, c3, l1, l2])

        pacman = geom.add_plane_surface(ll)

        # test setting physical groups
        geom.add_physical(p1, label="c")
        geom.add_physical(c0, label="arc")
        geom.add_physical(pacman, "dummy")
        geom.add_physical(pacman, label="77")

        mesh = geom.generate_mesh()

    ref = 54.312974717523744
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("pacman.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_max = 0.05

        rectangle = geom.add_rectangle([-1.0, -1.0, 0.0], 2.0, 2.0, corner_radius=0.2)
        disk1 = geom.add_disk([-1.2, 0.0, 0.0], 0.5)
        disk2 = geom.add_disk([+1.2, 0.0, 0.0], 0.5, 0.3)

        disk3 = geom.add_disk([0.0, -0.9, 0.0], 0.5)
        disk4 = geom.add_disk([0.0, +0.9, 0.0], 0.5)
        flat = geom.boolean_difference(
            geom.boolean_union([rectangle, disk1, disk2]),
            geom.boolean_union([disk3, disk4]),
        )
        geom.extrude(flat, [0, 0, 0.3])
        mesh = geom.generate_mesh()

    ref = 1.1742114942
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


def test2():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_max = 1.0

        mesh_size = 1
        h = 25
        w = 10
        length = 100
        # x_fin = -0.5 * length
        cr = 1

        f = 0.5 * w
        y = [-f, -f + cr, +f - cr, +f]
        z = [0.0, h - cr, h]
        f = 0.5 * cr
        x = [-f, f]
        points = [
            geom.add_point((x[0], y[0], z[0]), mesh_size=mesh_size),
            geom.add_point((x[0], y[0], z[1]), mesh_size=mesh_size),
            geom.add_point((x[0], y[1], z[1]), mesh_size=mesh_size),
            geom.add_point((x[0], y[1], z[2]), mesh_size=mesh_size),
            geom.add_point((x[0], y[2], z[2]), mesh_size=mesh_size),
            geom.add_point((x[0], y[2], z[1]), mesh_size=mesh_size),
            geom.add_point((x[0], y[3], z[1]), mesh_size=mesh_size),
            geom.add_point((x[0], y[3], z[0]), mesh_size=mesh_size),
        ]

        lines = [
            geom.add_line(points[0], points[1]),
            geom.add_circle_arc(points[1], points[2], points[3]),
            geom.add_line(points[3], points[4]),
            geom.add_circle_arc(points[4], points[5], points[6]),
            geom.add_line(points[6], points[7]),
            geom.add_line(points[7], points[0]),
        ]

        curve_loop = geom.add_curve_loop(lines)
        surface = geom.add_plane_surface(curve_loop)
        geom.extrude(surface, translation_axis=[length, 0, 0])

        mesh = geom.generate_mesh()

    ref = 24941.503891355664
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("occ_extrude.vtu")



import random

import numpy as np
from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        # Generate an approximation of a circle
        t = np.arange(0, 2.0 * np.pi, 0.05)
        x = np.column_stack([np.cos(t), np.sin(t), np.zeros_like(t)])
        points = [geom.add_point(p) for p in x]

        # Shuffle the orientation of lines by point order
        o = [0 if k % 3 == 0 else 1 for k in range(len(points))]

        lines = [
            geom.add_line(points[k + o[k]], points[k + (o[k] + 1) % 2])
            for k in range(len(points) - 1)
        ]
        lines.append(geom.add_line(points[-1], points[0]))

        # Shuffle the order of lines
        random.seed(1)
        random.shuffle(lines)

        oriented_lines = pygmsh.orient_lines(lines)
        ll = geom.add_curve_loop(oriented_lines)
        geom.add_plane_surface(ll)

        mesh = geom.generate_mesh()

    ref = np.pi
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("physical.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        lcar = 0.1
        circle = geom.add_circle([0.5, 0.5, 0.0], 1.0, lcar)
        triangle = geom.add_polygon(
            [[2.0, -0.5, 0.0], [4.0, -0.5, 0.0], [4.0, 1.5, 0.0]], lcar
        )
        rectangle = geom.add_rectangle(4.75, 6.25, -0.24, 1.25, 0.0, lcar)
        # hold all domain
        geom.add_polygon(
            [
                [-1.0, -1.0, 0.0],
                [+7.0, -1.0, 0.0],
                [+7.0, +2.0, 0.0],
                [-1.0, +2.0, 0.0],
            ],
            lcar,
            holes=[circle.curve_loop, triangle.curve_loop, rectangle.curve_loop],
        )
        mesh = geom.generate_mesh()

    ref = 24.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("subdomains.vtu")



"""Test module for boolean operations."""
import meshio
import numpy as np
import pytest
from helpers import compute_volume

import pygmsh


def square_loop(geom):
    """Construct square using built in geometry."""
    points = [
        geom.add_point([-0.5, -0.5], 0.05),
        geom.add_point([-0.5, 0.5], 0.05),
        geom.add_point([0.5, 0.5], 0.05),
        geom.add_point([0.5, -0.5], 0.05),
    ]
    lines = [
        geom.add_line(points[0], points[1]),
        geom.add_line(points[1], points[2]),
        geom.add_line(points[2], points[3]),
        geom.add_line(points[3], points[0]),
    ]
    return geom.add_curve_loop(lines)


def circle_loop(geom):
    """construct circle using geo geometry module."""
    points = [
        geom.add_point([+0.0, +0.0], 0.05),
        geom.add_point([+0.0, +0.1], 0.05),
        geom.add_point([-0.1, +0.0], 0.05),
        geom.add_point([+0.0, -0.1], 0.05),
        geom.add_point([+0.1, +0.0], 0.05),
    ]
    quarter_circles = [
        geom.add_circle_arc(points[1], points[0], points[2]),
        geom.add_circle_arc(points[2], points[0], points[3]),
        geom.add_circle_arc(points[3], points[0], points[4]),
        geom.add_circle_arc(points[4], points[0], points[1]),
    ]
    return geom.add_curve_loop(quarter_circles)


def _square_hole_classical(geom):
    """Construct surface using builtin and boolean methods."""
    # construct surface with hole using standard built in
    geom.characteristic_length_min = 0.05
    geom.characteristic_length_max = 0.05
    square = square_loop(geom)
    circle = circle_loop(geom)
    geom.add_plane_surface(square, [circle])


def _square_hole_cad(geom):
    # construct surface using boolean
    geom.characteristic_length_min = 0.05
    geom.characteristic_length_max = 0.05
    square2 = square_loop(geom)
    curve_loop2 = circle_loop(geom)
    surf1 = geom.add_plane_surface(square2)
    surf2 = geom.add_plane_surface(curve_loop2)
    geom.boolean_difference(surf1, surf2)


@pytest.mark.parametrize("fun", [_square_hole_classical, _square_hole_cad])
def test_square_circle_hole(fun):
    """Test planar surface with holes.

    Construct it with boolean operations and verify that it is the same.
    """
    with pygmsh.occ.Geometry() as geom:
        fun(geom)
        mesh = geom.generate_mesh()
    surf = 1 - 0.1 ** 2 * np.pi
    assert np.abs((compute_volume(mesh) - surf) / surf) < 1e-3


@pytest.mark.skip()
def test_square_circle_slice():
    """Test planar surface square with circular hole.

    Also test for surface area of fragments.
    """
    with pygmsh.occ.Geometry() as geom:
        square = square_loop(geom)
        curve_loop = circle_loop(geom)
        surf1 = geom.add_plane_surface(square)
        surf2 = geom.add_plane_surface(curve_loop)
        geom.boolean_fragments(surf1, surf2)
        mesh = geom.generate_mesh()

    ref = 1.0
    val = compute_volume(mesh)
    assert np.abs(val - ref) < 1e-3 * ref

    # Gmsh 4 default format MSH4 doesn't have geometrical entities.
    outer_mask = np.where(mesh.cell_data["gmsh:geometrical"][2] == 13)[0]
    outer_cells = {}
    outer_cells["triangle"] = mesh.cells_dict["triangle"][outer_mask]

    inner_mask = np.where(mesh.cell_data["gmsh:geometrical"][2] == 12)[0]
    inner_cells = {}
    inner_cells["triangle"] = mesh.cells_dict["triangle"][inner_mask]

    ref = 1 - 0.1 ** 2 * np.pi
    value = compute_volume(meshio.Mesh(mesh.points, outer_cells))
    assert np.abs(value - ref) < 1e-2 * ref


@pytest.mark.skip("cell data not working yet")
def test_fragments_diff_union():
    """Test planar surface with holes.

    Construct it with boolean operations and verify that it is the same.
    """
    # construct surface using boolean
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.04
        geom.characteristic_length_max = 0.04
        square = square_loop(geom)
        surf1 = geom.add_plane_surface(square)
        curve_loop = circle_loop(geom)
        surf2 = geom.add_plane_surface(curve_loop)

        geom.add_physical([surf1], label="1")
        geom.add_physical([surf2], label="2")
        geom.boolean_difference(surf1, surf2, delete_other=False)
        mesh = geom.generate_mesh()

    ref = 1.0
    assert np.abs(compute_volume(mesh) - ref) < 1e-3 * ref

    surf = 1 - 0.1 ** 2 * np.pi
    outer_mask = np.where(mesh.cell_data_dict["gmsh:geometrical"]["triangle"] == 1)[0]
    outer_cells = {}
    outer_cells["triangle"] = mesh.cells_dict["triangle"][outer_mask]

    inner_mask = np.where(mesh.cell_data_dict["gmsh:geometrical"]["triangle"] == 2)[0]
    inner_cells = {}
    inner_cells["triangle"] = mesh.cells_dict["triangle"][inner_mask]

    value = compute_volume(meshio.Mesh(mesh.points, outer_cells))
    assert np.abs(value - surf) < 1e-2 * surf


@pytest.mark.skip("cell data not working yet")
def test_diff_physical_assignment():
    """construct surface using boolean.

    Ensure that after a difference operation the initial volume physical label
    is kept for the operated geometry.
    """
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.05
        geom.characteristic_length_max = 0.05
        square2 = square_loop(geom)
        curve_loop2 = circle_loop(geom)
        surf1 = geom.add_plane_surface(square2)
        surf2 = geom.add_plane_surface(curve_loop2)
        geom.add_physical(surf1, label="1")
        geom.boolean_difference(surf1, surf2)
        mesh = geom.generate_mesh()
    assert np.allclose(
        mesh.cell_data_dict["gmsh:geometrical"]["triangle"],
        np.ones(mesh.cells_dict["triangle"].shape[0]),
    )
    surf = 1 - 0.1 ** 2 * np.pi
    assert np.abs((compute_volume(mesh) - surf) / surf) < 1e-3


def test_polygon_diff():
    with pygmsh.occ.Geometry() as geom:
        poly = geom.add_polygon([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0]])
        disk = geom.add_disk([0, 0, 0], 0.5)
        geom.boolean_difference(poly, disk)


def test_mesh_size_removal():
    with pygmsh.occ.Geometry() as geom:
        box0 = geom.add_box([0.0, 0, 0], [1, 1, 1], mesh_size=0.1)
        box1 = geom.add_box([0.5, 0.5, 1], [0.5, 0.5, 1], mesh_size=0.2)
        geom.boolean_union([box0, box1])
        geom.generate_mesh()


if __name__ == "__main__":
    test_square_circle_slice()



"""
Creates a mesh for a square with a round hole.
"""
from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        circle = geom.add_circle(
            x0=[0.5, 0.5, 0.0],
            radius=0.25,
            mesh_size=0.1,
            num_sections=4,
            make_surface=False,
        )
        geom.add_rectangle(
            0.0, 1.0, 0.0, 1.0, 0.0, mesh_size=0.1, holes=[circle.curve_loop]
        )
        mesh = geom.generate_mesh()

    ref = 0.8086582838174551
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("rectangle_with_hole.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        container = geom.add_rectangle([0.0, 0.0, 0.0], 10.0, 10.0)

        letter_i = geom.add_rectangle([2.0, 2.0, 0.0], 1.0, 4.5)
        i_dot = geom.add_disk([2.5, 7.5, 0.0], 0.6)

        disk1 = geom.add_disk([6.25, 4.5, 0.0], 2.5)
        disk2 = geom.add_disk([6.25, 4.5, 0.0], 1.5)
        letter_o = geom.boolean_difference(disk1, disk2)

        geom.boolean_difference(
            container, geom.boolean_union([letter_i, i_dot, letter_o])
        )

        mesh = geom.generate_mesh()

    ref = 81.9131851877
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    # import meshio
    # meshio.write_points_cells('m.vtu', *test())
    from helpers import plot

    mesh = test()
    plot("meshio_logo.png", mesh.points, mesh.get_cells_type("triangle"))



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_max = 0.1
        p0 = geom.add_point([-0.5, -0.5, 0], 0.01)
        p1 = geom.add_point([+0.5, -0.5, 0], 0.01)
        p2 = geom.add_point([+0.5, +0.5, 0], 0.01)
        p3 = geom.add_point([-0.5, +0.5, 0], 0.01)
        l0 = geom.add_line(p0, p1)
        l1 = geom.add_line(p1, p2)
        l2 = geom.add_line(p2, p3)
        l3 = geom.add_line(p3, p0)
        ll0 = geom.add_curve_loop([l0, l1, l2, l3])
        square_builtin = geom.add_plane_surface(ll0)
        square_occ = geom.add_rectangle([0, 0, 0], 1.0, 1.0)
        geom.boolean_difference(square_occ, square_builtin)
        mesh = geom.generate_mesh()

    ref = 0.75
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("mix.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        lcar = 0.1
        p1 = geom.add_point([0.0, 0.0, 0.0], lcar)
        p2 = geom.add_point([1.0, 0.0, 0.0], lcar)
        p3 = geom.add_point([1.0, 0.5, 0.0], lcar)
        p4 = geom.add_point([1.0, 1.0, 0.0], lcar)
        s1 = geom.add_spline([p1, p2, p3, p4])

        p2 = geom.add_point([0.0, 1.0, 0.0], lcar)
        p3 = geom.add_point([0.5, 1.0, 0.0], lcar)
        s2 = geom.add_spline([p4, p3, p2, p1])

        ll = geom.add_curve_loop([s1, s2])
        geom.add_plane_surface(ll)

        mesh = geom.generate_mesh()
    ref = 1.0809439490373247
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("splines.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        rectangle = geom.add_rectangle(0.0, 1.0, 0.0, 1.0, 0.0, 0.1)
        geom.set_recombined_surfaces([rectangle.surface])
        mesh = geom.generate_mesh(dim=2)

    ref = 1.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("quads.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        poly = geom.add_polygon(
            [[0.0, 0.5], [1.0, 0.5], [1.0, 1.0], [0.0, 1.0]],
            mesh_size=0.05,
        )
        cp = geom.copy(poly)
        geom.symmetrize(cp, [0.0, 1.0, 0.0, -0.5])
        mesh = geom.generate_mesh()

    ref = 1.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("symmetry.vtk")



from itertools import permutations

import meshio
from helpers import compute_volume

import pygmsh


def test(lcar=1.0):
    with pygmsh.geo.Geometry() as geom:
        lbw = [2, 3, 5]
        points = [geom.add_point([x, 0.0, 0.0], lcar) for x in [0.0, lbw[0]]]
        line = geom.add_line(*points)

        _, rectangle, _ = geom.extrude(
            line, translation_axis=[0.0, lbw[1], 0.0], num_layers=lbw[1], recombine=True
        )
        geom.extrude(
            rectangle,
            translation_axis=[0.0, 0.0, lbw[2]],
            num_layers=lbw[2],
            recombine=True,
        )
        # compute_volume only supports 3D for tetras, but does return surface area for
        # quads
        mesh = geom.generate_mesh()
        # mesh.remove_lower_dimensional_cells()
        # mesh.remove_orphaned_nodes()

    ref = sum(l * w for l, w in permutations(lbw, 2))  # surface area
    # TODO compute hex volumes
    quad_mesh = meshio.Mesh(mesh.points, {"quad": mesh.cells_dict["quad"]})
    assert abs(compute_volume(quad_mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    meshio.write("hex.vtu", test())



from helpers import compute_volume

import pygmsh


def test(mesh_size=0.05):
    with pygmsh.geo.Geometry() as geom:
        # Draw a cross with a circular hole
        circ = geom.add_circle(
            [0.0, 0.0, 0.0], 0.1, mesh_size=mesh_size, make_surface=False
        )
        poly = geom.add_polygon(
            [
                [+0.0, +0.5, 0.0],
                [-0.1, +0.1, 0.0],
                [-0.5, +0.0, 0.0],
                [-0.1, -0.1, 0.0],
                [+0.0, -0.5, 0.0],
                [+0.1, -0.1, 0.0],
                [+0.5, +0.0, 0.0],
                [+0.1, +0.1, 0.0],
            ],
            mesh_size=mesh_size,
            holes=[circ],
        )
        axis = [0, 0, 1.0]
        geom.extrude(poly, translation_axis=axis, num_layers=1)
        mesh = geom.generate_mesh()

    ref = 0.16951514066385628
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("layers.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        # test setters, getters
        print(geom.characteristic_length_min)
        print(geom.characteristic_length_max)
        geom.characteristic_length_min = 2.0
        geom.characteristic_length_max = 2.0

        rect1 = geom.add_rectangle([10.0, 0.0, 0.0], 20.0, 40.0, corner_radius=5.0)
        rect2 = geom.add_rectangle([0.0, 10.0, 0.0], 40.0, 20.0, corner_radius=5.0)
        disk1 = geom.add_disk([14.5, 35.0, 0.0], 1.85)
        disk2 = geom.add_disk([25.5, 5.0, 0.0], 1.85)

        rect3 = geom.add_rectangle([10.0, 30.0, 0.0], 10.0, 1.0)
        rect4 = geom.add_rectangle([20.0, 9.0, 0.0], 10.0, 1.0)

        r1 = geom.add_rectangle([9.0, 0.0, 0.0], 21.0, 20.5, corner_radius=8.0)
        r2 = geom.add_rectangle([10.0, 00.0, 0.0], 20.0, 19.5, corner_radius=7.0)
        diff1 = geom.boolean_difference(r1, r2)
        r22 = geom.add_rectangle([9.0, 10.0, 0.0], 11.0, 11.0)
        inter1 = geom.boolean_intersection([diff1, r22])

        r3 = geom.add_rectangle([10.0, 19.5, 0.0], 21.0, 21.0, corner_radius=8.0)
        r4 = geom.add_rectangle([10.0, 20.5, 0.0], 20.0, 20.0, corner_radius=7.0)
        diff2 = geom.boolean_difference(r3, r4)
        r33 = geom.add_rectangle([20.0, 19.0, 0.0], 11.0, 11.0)
        inter2 = geom.boolean_intersection([diff2, r33])

        geom.boolean_difference(
            geom.boolean_union([rect1, rect2]),
            geom.boolean_union([disk1, disk2, rect3, rect4, inter1, inter2]),
        )

        mesh = geom.generate_mesh()
    ref = 1082.4470502181903
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    mesh = test()
    points = mesh.points
    cells = mesh.get_cells_type("triangle")

    # import optimesh

    # # points, cells = optimesh.cvt.quasi_newton_uniform_lloyd(
    # #     points, cells, 1.0e-5, 1000, omega=2.0, verbose=True
    # # )
    # # points, cells = optimesh.cvt.quasi_newton_uniform_blocks(
    # #     points, cells, 1.0e-5, 1000, verbose=True
    # # )
    # points, cells = optimesh.cvt.quasi_newton_uniform_full(
    #     points, cells, 1.0e-5, 1000, verbose=True
    # )

    # # from helpers import plot
    # # plot("logo.png", points, {"triangle": cells})
    import meshio

    # meshio.write_points_cells("logo.vtu", points, {"triangle": cells})
    mesh = meshio.Mesh(points, {"triangle": cells})
    meshio.svg.write(
        "logo.svg", mesh, float_fmt=".3f", stroke_width="1", force_width=300
    )



import numpy as np
from helpers import compute_volume

import pygmsh


def test(mesh_size=0.05):
    with pygmsh.geo.Geometry() as geom:
        # Draw a cross with a circular hole
        circ = geom.add_circle([0.0, 0.0], 0.1, mesh_size=mesh_size)
        poly = geom.add_polygon(
            [
                [+0.0, +0.5],
                [-0.1, +0.1],
                [-0.5, +0.0],
                [-0.1, -0.1],
                [+0.0, -0.5],
                [+0.1, -0.1],
                [+0.5, +0.0],
                [+0.1, +0.1],
            ],
            mesh_size=mesh_size,
            holes=[circ],
        )

        geom.twist(
            poly,
            translation_axis=[0.0, 0.0, 1.0],
            rotation_axis=[0.0, 0.0, 1.0],
            point_on_axis=[0.0, 0.0, 0.0],
            angle=2.0 / 6.0 * np.pi,
        )
        mesh = geom.generate_mesh()

    ref = 0.16951514066385628
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("screw.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.add_box([0.0, 0.0, 0.0], [1, 2, 3], mesh_size=0.1)
        mesh = geom.generate_mesh()

    ref = 6.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("occ_box.vtu")



import pytest
from helpers import compute_volume

import pygmsh


def test_in_surface():
    with pygmsh.geo.Geometry() as geom:
        poly = geom.add_polygon(
            [
                [0, 0.3],
                [0, 1.1],
                [0.9, 1.1],
                [0.9, 0.3],
                [0.6, 0.7],
                [0.3, 0.7],
                [0.2, 0.4],
            ],
            mesh_size=[0.2, 0.2, 0.2, 0.2, 0.03, 0.03, 0.01],
        )
        geom.in_surface(poly.lines[4], poly)
        geom.in_surface(poly.points[6], poly)
        mesh = geom.generate_mesh()

    ref = 0.505
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


# Exception: PLC Error:  A segment and a facet intersect at point
@pytest.mark.skip
def test_in_volume():
    with pygmsh.geo.Geometry() as geom:
        box = geom.add_box(-1, 2, -1, 2, 0, 1, mesh_size=0.5)
        poly = geom.add_polygon(
            [
                [0.0, 0.3],
                [0.0, 1.1],
                [0.9, 1.1],
                [0.9, 0.3],
                [0.6, 0.7],
                [0.3, 0.7],
                [0.2, 0.4],
            ],
            mesh_size=[0.2, 0.2, 0.2, 0.2, 0.03, 0.03, 0.01],
        )
        geom.in_volume(poly.lines[4], box.volume)
        geom.in_volume(poly.points[6], box.volume)
        geom.in_volume(poly, box.volume)

        mesh = geom.generate_mesh()
    ref = 30.505
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test_in_surface().write("test.vtk")



from math import sqrt

import pytest

import pygmsh


@pytest.mark.skip("Only works in Gmsh 4.7.0+")
def test():
    with pygmsh.occ.Geometry() as geom:
        geom.add_ball([0.0, 0.0, 0.0], 1.0)
        geom.set_mesh_size_callback(
            lambda dim, tag, x, y, z: abs(sqrt(x ** 2 + y ** 2 + z ** 2) - 0.5) + 0.1
        )
        mesh = geom.generate_mesh()

    assert mesh.cells[0].data.shape[0] > 1500



import numpy as np
from helpers import compute_volume

import pygmsh


def test(irad=0.05, orad=0.6):
    """Torus, rotated in space."""
    with pygmsh.geo.Geometry() as geom:
        R = pygmsh.rotation_matrix([1.0, 0.0, 0.0], np.pi / 2)
        geom.add_torus(irad=irad, orad=orad, mesh_size=0.03, x0=[0.0, 0.0, -1.0], R=R)
        mesh = geom.generate_mesh()

    ref = 2 * np.pi ** 2 * orad * irad ** 2
    assert np.isclose(compute_volume(mesh), ref, rtol=5e-2)
    return mesh


if __name__ == "__main__":
    test().write("torus.vtu")



"""Creates a mesh on a cube.
"""
from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        geom.add_box(0, 1, 0, 1, 0, 1, 1.0)
        mesh = geom.generate_mesh()

    ref = 1.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("cube.vtu")



import meshio

import pygmsh


def test(lcar=0.5):
    with pygmsh.geo.Geometry() as geom:
        poly = geom.add_polygon([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0]], lcar)

        top, volume, lat = geom.extrude(poly, [0, 0, 2])

        geom.add_physical(poly, label="bottom")
        geom.add_physical(top, label="top")
        geom.add_physical(volume, label="volume")
        geom.add_physical(lat, label="lat")
        geom.add_physical(poly.lines[0], label="line")

        mesh = geom.generate_mesh()
        assert len(mesh.cell_sets) == 5
    return mesh


if __name__ == "__main__":
    test().write("physical.vtu")
    read_mesh = meshio.read("physical.vtu")
    assert len(read_mesh.cell_sets) == 5



"""
Creates a mesh for an ellipsoid.
"""
from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        geom.add_ellipsoid([0.0, 0.0, 0.0], [1.0, 0.5, 0.75], 0.05)
        mesh = geom.generate_mesh()
    ref = 1.5676038497587947
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("ellipsoid.vtu")



import numpy as np
from helpers import compute_volume

import pygmsh


def test(radius=1.0):
    with pygmsh.geo.Geometry() as geom:
        R = [
            pygmsh.rotation_matrix(np.eye(1, 3, d)[0], theta)
            for d, theta in enumerate(np.pi / np.array([2.0, 3.0, 5]))
        ]
        geom.add_circle([7.0, 11.0, 13.0], radius, 0.1, R[0] @ R[1] @ R[2])
        ref = np.pi * radius ** 2
        mesh = geom.generate_mesh()

    assert np.isclose(compute_volume(mesh), ref, rtol=1e-2)
    return mesh


if __name__ == "__main__":
    test().write("circle_transformed.vtk")



"""Test translation for all dimensions."""
import numpy as np
from helpers import compute_volume

import pygmsh

# def test_translation1d():
#     """Translation of a line."""
#     geom = pygmsh.geo.Geometry()
#     points = []
#     for array in [[1, 0, 0], [0, 0, 0], [0, 1, 0]]:
#         points.append(geom.add_point(array, 0.5))
#     circle = geom.add_circle_arc(*points)
#     # mesh = geom.generate_mesh()
#     geom.translate(circle, [1.5, 0, 0])
#     translated_mesh = geom.generate_mesh()
#     points[:, 0] = points[:, 0] + 1.5
#     assert np.allclose(points, translated_mesh.points)


def test_translation2d():
    """Translation of a surface object."""
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.05
        geom.characteristic_length_max = 0.05
        disk = geom.add_disk([0, 0, 0], 1)
        disk2 = geom.add_disk([1.5, 0, 0], 1)
        geom.translate(disk, [1.5, 0, 0])
        geom.boolean_union([disk2, disk])
        mesh = geom.generate_mesh()
    surf = np.pi
    assert np.abs(compute_volume(mesh) - surf) < 1e-3 * surf


def test_translation3d():
    """Translation of a volume object."""
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.2
        geom.characteristic_length_max = 0.2
        ball = geom.add_ball([0, 0, 0], 1)
        ball2 = geom.add_ball([1.5, 0, 0], 1)
        geom.translate(ball, [1.5, 0, 0])
        geom.boolean_union([ball2, ball])
        mesh = geom.generate_mesh()
    surf = 4 / 3 * np.pi
    assert np.abs(compute_volume(mesh) - surf) < 2e-2 * surf


if __name__ == "__main__":
    # test_translation1d()
    test_translation2d()
    test_translation3d()



import math

from helpers import compute_volume

import pygmsh


def test_union():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.1
        geom.characteristic_length_max = 0.1
        rectangle = geom.add_rectangle([-1.0, -1.0, 0.0], 2.0, 2.0)
        disk_w = geom.add_disk([-1.0, 0.0, 0.0], 0.5)
        disk_e = geom.add_disk([+1.0, 0.0, 0.0], 0.5)
        geom.boolean_union([rectangle, disk_w, disk_e])
        mesh = geom.generate_mesh()

    ref = 4.780361
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


def test_intersection():
    with pygmsh.occ.Geometry() as geom:
        angles = [math.pi * 3 / 6, math.pi * 7 / 6, math.pi * 11 / 6]
        disks = [
            geom.add_disk([math.cos(angles[0]), math.sin(angles[0]), 0.0], 1.5),
            geom.add_disk([math.cos(angles[1]), math.sin(angles[1]), 0.0], 1.5),
            geom.add_disk([math.cos(angles[2]), math.sin(angles[2]), 0.0], 1.5),
        ]
        geom.boolean_intersection(disks)
        mesh = geom.generate_mesh()

    ref = 1.0290109753807914
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


def test_difference():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.1
        geom.characteristic_length_max = 0.1
        rectangle = geom.add_rectangle([-1.0, -1.0, 0.0], 2.0, 2.0)
        disk_w = geom.add_disk([-1.0, 0.0, 0.0], 0.5)
        disk_e = geom.add_disk([+1.0, 0.0, 0.0], 0.5)
        geom.boolean_union([disk_w, disk_e])
        geom.boolean_difference(rectangle, geom.boolean_union([disk_w, disk_e]))
        mesh = geom.generate_mesh()

    ref = 3.2196387
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


def test_all():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.1
        geom.characteristic_length_max = 0.1

        rectangle = geom.add_rectangle([-1.0, -1.0, 0.0], 2.0, 2.0)
        disk1 = geom.add_disk([-1.0, 0.0, 0.0], 0.5)
        disk2 = geom.add_disk([+1.0, 0.0, 0.0], 0.5)
        union = geom.boolean_union([rectangle, disk1, disk2])

        disk3 = geom.add_disk([0.0, -1.0, 0.0], 0.5)
        disk4 = geom.add_disk([0.0, +1.0, 0.0], 0.5)
        geom.boolean_difference(union, geom.boolean_union([disk3, disk4]))
        mesh = geom.generate_mesh()

    ref = 4.0
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test_difference().write("boolean.vtu")



import pygmsh


def test(lcar=0.1):
    with pygmsh.geo.Geometry() as geom:
        poly = geom.add_polygon(
            [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [1.0, 1.0, 0.0], [0.0, 1.0, 0.0]], lcar
        )
        geom.set_transfinite_surface(poly, "Left", corner_pts=[])
        mesh = geom.generate_mesh()
    assert len(mesh.cells_dict["triangle"]) == 10 * 10 * 2
    return mesh


if __name__ == "__main__":
    test().write("transfinite.vtu")



import numpy as np
from helpers import compute_volume

import pygmsh


def test():
    X0 = np.array(
        [[+0.0, +0.0, 0.0], [+0.5, +0.3, 0.1], [-0.5, +0.3, 0.1], [+0.5, -0.3, 0.1]]
    )
    R = np.array([0.1, 0.2, 0.1, 0.14])

    with pygmsh.geo.Geometry() as geom:
        holes = [
            geom.add_ball(x0, r, with_volume=False, mesh_size=0.2 * r).surface_loop
            for x0, r in zip(X0, R)
        ]
        # geom.add_box(
        #         -1, 1,
        #         -1, 1,
        #         -1, 1,
        #         mesh_size=0.2,
        #         holes=holes
        #         )
        geom.add_ball([0, 0, 0], 1.0, mesh_size=0.2, holes=holes)
        # geom.add_physical_volume(ball, label="cheese")
        mesh = geom.generate_mesh(algorithm=5)

    ref = 4.07064892966291
    assert abs(compute_volume(mesh) - ref) < 2.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("swiss_cheese.vtu")



from helpers import compute_volume

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        geom.add_circle(
            [0.0, 0.0, 0.0],
            1.0,
            mesh_size=0.1,
            num_sections=4,
            # If compound==False, the section borders have to be points of the
            # discretization. If using a compound circle, they don't; gmsh can
            # choose by itself where to point the circle points.
            compound=True,
        )
        # geom.add_physical(c.plane_surface, "super disk")
        mesh = geom.generate_mesh()

    ref = 3.1363871677682247
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("circle.vtk")



import numpy as np

import pygmsh


def test():
    # Characteristic length
    lcar = 1e-1

    # Coordinates of lower-left and upper-right vertices of a square domain
    xmin = 0.0
    xmax = 5.0
    ymin = 0.0
    ymax = 5.0

    # Vertices of a square hole
    squareHoleCoordinates = np.array([[1.0, 1.0], [4.0, 1.0], [4.0, 4.0], [1.0, 4.0]])

    with pygmsh.geo.Geometry() as geom:
        # Create square hole
        squareHole = geom.add_polygon(squareHoleCoordinates, lcar, make_surface=False)
        # Create square domain with square hole
        geom.add_rectangle(
            xmin, xmax, ymin, ymax, 0.0, lcar, holes=[squareHole.curve_loop]
        )
        mesh = geom.generate_mesh(order=2)
    # TODO support for volumes of triangle6
    # ref = 16.0
    # from helpers import compute_volume
    # assert abs(compute_volume(points, cells) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("hole_in_square.vtu")



import pygmsh


def test():
    with pygmsh.occ.Geometry() as geom:
        geom.characteristic_length_min = 0.1
        geom.characteristic_length_max = 0.1

        ball = geom.add_ball([0.0, 0.0, 0.0], 1.0)
        box1 = geom.add_box([0, 0, 0], [1, 1, 1])
        box2 = geom.add_box([-2, -0.5, -0.5], [1.5, 0.8, 0.8])

        cut = geom.boolean_difference(ball, box1)
        frag = geom.boolean_fragments(cut, box2)

        # The three fragments are:
        # frag[0]: The ball with two cuts
        # frag[1]: The intersection of the stick and the ball
        # frag[2]: The stick without the ball
        geom.add_physical([frag[0], frag[1]], label="Sphere cut by box 1")
        geom.add_physical(frag[2], label="Box 2 cut by sphere")

        mesh = geom.generate_mesh(algorithm=6)

    assert "Sphere cut by box 1" in mesh.cell_sets
    assert "Box 2 cut by sphere" in mesh.cell_sets

    # mesh.remove_lower_dimensional_cells()
    # mesh.sets_to_int_data()
    return mesh


if __name__ == "__main__":
    test().write("ball-with-stick.vtu")



import numpy as np
from helpers import compute_volume

import pygmsh


def test():
    """Pipe with double-ring enclosure, rotated in space."""
    with pygmsh.geo.Geometry() as geom:
        sqrt2on2 = 0.5 * np.sqrt(2.0)
        R = pygmsh.rotation_matrix([sqrt2on2, sqrt2on2, 0], np.pi / 6.0)
        geom.add_pipe(
            inner_radius=0.3, outer_radius=0.4, length=1.0, R=R, mesh_size=0.04
        )

        R = np.array([[0.0, 0.0, 1.0], [0.0, 1.0, 0.0], [1.0, 0.0, 0.0]])
        geom.add_pipe(
            inner_radius=0.3,
            outer_radius=0.4,
            length=1.0,
            mesh_size=0.04,
            R=R,
            variant="circle_extrusion",
        )
        mesh = geom.generate_mesh()

    ref = 0.43988203517453256
    assert abs(compute_volume(mesh) - ref) < 1.0e-2 * ref
    return mesh


if __name__ == "__main__":
    test().write("pipes.vtu")



import pytest

import pygmsh


@pytest.mark.skip()
def test():
    with pygmsh.occ.Geometry() as geom:
        geom.add_ball([0.0, 0.0, 0.0], 1.0, mesh_size=0.1)
        mesh = geom.generate_mesh()

    pygmsh.optimize(mesh)



"""Test translation for all dimensions."""
import numpy as np

import pygmsh


def test_rotation2d():
    """Rotation of a surface object."""
    angle = np.pi / 5

    # Generate reference geometry
    with pygmsh.geo.Geometry() as geom:
        rect = geom.add_rectangle(0.0, 2.0, 0.0, 1.0, 0.0, 0.1)
        mesh_unrot = geom.generate_mesh()
    vertex_index = mesh_unrot.cells_dict["vertex"]
    vertex_index = vertex_index.reshape((vertex_index.shape[0],))

    with pygmsh.geo.Geometry() as geom:
        # Generate rotated geometry
        geom = pygmsh.geo.Geometry()
        rect = geom.add_rectangle(0.0, 2.0, 0.0, 1.0, 0.0, 0.1)
        geom.rotate(rect.surface, (0, 0, 0), angle, (0, 0, 1))
        mesh = geom.generate_mesh()

    new_vertex_index = mesh.cells_dict["vertex"]
    new_vertex_index = new_vertex_index.reshape((new_vertex_index.shape[0],))

    # Generate rotation matrix and compare with rotated geometry
    Rm = pygmsh.helpers.rotation_matrix([0, 0, 1], angle)
    for v, v_new in zip(vertex_index, new_vertex_index):
        point = mesh_unrot.points[v, :]
        rot_point = np.dot(Rm, point)
        new_point = mesh.points[v, :]
        assert np.allclose(rot_point, new_point)


if __name__ == "__main__":
    test_rotation2d()



import numpy as np

import pygmsh


def test():
    with pygmsh.geo.Geometry() as geom:
        pts = [
            geom.add_point((0.0, 0.0, 0.0), mesh_size=1.0),
            geom.add_point((2.0, 0.0, 0.0), mesh_size=1.0),
            geom.add_point((0.0, 1.0, 0.0), mesh_size=1.0),
            geom.add_point((2.0, 1.0, 0.0), mesh_size=1.0),
        ]
        lines = [
            geom.add_line(pts[0], pts[1]),
            geom.add_line(pts[1], pts[3]),
            geom.add_line(pts[3], pts[2]),
            geom.add_line(pts[2], pts[0]),
        ]
        ll0 = geom.add_curve_loop(lines)
        rs0 = geom.add_surface(ll0)

        geom.set_transfinite_curve(lines[3], 3, "Progression", 1.0)
        geom.set_transfinite_curve(lines[1], 3, "Progression", 1.0)
        geom.set_transfinite_curve(lines[2], 3, "Progression", 1.0)
        geom.set_transfinite_curve(lines[0], 3, "Progression", 1.0)
        geom.set_transfinite_surface(rs0, "Left", pts)
        geom.set_recombined_surfaces([rs0])

        mesh = geom.generate_mesh()

    assert "quad" in mesh.cells_dict.keys()
    ref = np.array([[0, 4, 8, 7], [7, 8, 6, 2], [4, 1, 5, 8], [8, 5, 3, 6]])
    assert np.array_equal(ref, mesh.cells_dict["quad"])
    return mesh


if __name__ == "__main__":
    test().write("rectangle_structured.vtu")
